( cd usr/share/doc/HTML/en/kmldonkey ; rm -rf common )
( cd usr/share/doc/HTML/en/kmldonkey ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/lib ; rm -rf libkmldonkey.so )
( cd usr/lib ; ln -sf libkmldonkey.so.4.0.0 libkmldonkey.so )
( cd usr/lib ; rm -rf libkmldonkey.so.4 )
( cd usr/lib ; ln -sf libkmldonkey.so.4.0.0 libkmldonkey.so.4 )
( cd usr/lib ; rm -rf libkmldonkeytools.so )
( cd usr/lib ; ln -sf libkmldonkeytools.so.0.0.0 libkmldonkeytools.so )
( cd usr/lib ; rm -rf libkmldonkeytools.so.0 )
( cd usr/lib ; ln -sf libkmldonkeytools.so.0.0.0 libkmldonkeytools.so.0 )
